@@ -16,7 +16,7 @@ from random import choice
 from xml.dom.minidom import parseString
 
 # solrpy
-from solr import SolrConnection
+from solr import SolrConnection, SolrPaginator
 
 SOLR_PATH = "/solr"
 SOLR_HOST = "localhost"
@@ -444,7 +444,7 @@ class TestDocumentsDeletion(unittest.TestCase):
 class TestQuerying(unittest.TestCase):
 
     def setUp(self):
-        self.conn = SolrConnection(SOLR_HTTP)
+        self.conn = SolrConnection(SOLR_HTTP,timeout=1)
 
     def test_query_string(self):
         """ Get documents (all default fields) by a simple query.
@@ -847,7 +847,31 @@ class TestQuerying(unittest.TestCase):
         # solrpy 0.1
         self.assertTrue(str(query_timestamp.microsecond).startswith(microsecond))
         self.assertTrue(query_timestamp.microsecond/int(microsecond) == 1000)
-
+        
+    def test_facet_field(self):
+        """ Test basic facet fields and make sure they are included in the 
+        response properly """
+        
+        self.conn.delete_query('id:[* TO *]')
+        self.conn.optimize()
+        
+        for i in range(0,12):
+            self.conn.add(id=i,user_id=i%3,data=get_rand_string(),num=10)
+        
+        self.conn.optimize()
+        
+        results = self.conn.query('id:[* TO *]',facet='true',
+                                  facet_field=['user_id','num'])
+        
+        self.assertTrue(hasattr(results,'facet_counts'))
+        self.assertTrue(u'facet_fields' in results.facet_counts)
+        self.assertTrue(u'num' in results.facet_counts[u'facet_fields'])
+        self.assertTrue(u'user_id' in results.facet_counts[u'facet_fields'])
+        self.assertEqual(len(results.facet_counts[u'facet_fields'][u'num']),1)
+        self.assertEqual(len(results.facet_counts[u'facet_fields'][u'user_id']),3)
+        self.assertEqual(results.facet_counts[u'facet_fields'][u'num'],{u'10':12})
+        self.assertEqual(results.facet_counts[u'facet_fields'][u'user_id'],{u'0':4,u'1':4,u'2':4})
+        
     def tearDown(self):
         self.conn.close()
 
@@ -975,5 +999,78 @@ class TestResponse(unittest.TestCase):
         self.conn.close()
 
 
+class TestPaginator(unittest.TestCase):
+    
+    def setUp(self):
+        self.conn = SolrConnection(SOLR_HTTP)
+        self.conn.delete_query('*:*')
+        for i in range(0,15):
+            self.conn.add(id=i, data='data_%02i' % i)
+        self.conn.commit()
+        self.result = self.conn.query('*:*', sort='data', sort_order='desc')
+    
+    def test_num_pages(self):
+        """ Check the number of pages reported by the paginator """
+        paginator = SolrPaginator(self.result)
+        self.assertEqual(paginator.num_pages, 2)
+    
+    def test_count(self):
+        """ Check the result count reported by the paginator """
+        paginator = SolrPaginator(self.result)
+        self.assertEqual(paginator.count, 15)
+    
+    def test_page_range(self):
+        """ Check the page range returned by the paginator """
+        paginator = SolrPaginator(self.result)
+        self.assertEqual(paginator.page_range, [1,2])
+    
+    def test_default_page_size(self):
+        """ Test invalid/impproper default page sizes for paginator """
+        self.assertRaises(ValueError,SolrPaginator,self.result,'asdf')
+        self.assertRaises(ValueError,SolrPaginator,self.result,5)
+    
+    def test_page_one(self):
+        """ Test the first page from a paginator """
+        paginator = SolrPaginator(self.result)
+        page = paginator.page(1)
+        self.assertEqual(page.has_other_pages(), True)
+        self.assertEqual(page.has_next(), True)
+        self.assertEqual(page.has_previous(), False)
+        self.assertEqual(page.next_page_number(), 2)
+        self.assertEqual(page.start_index(), 0)
+        self.assertEqual(page.end_index(), 9)
+        self.assertEqual(len(page.object_list), 10)
+        self.assertEqual(page.object_list[0]['data'], 'data_14')
+    
+    def test_page_two(self):
+        """ Test the second/last page from a paginator """
+        paginator = SolrPaginator(self.result,default_page_size=10)
+        page = paginator.page(2)
+        self.assertEqual(page.has_other_pages(), True)
+        self.assertEqual(page.has_next(), False)
+        self.assertEqual(page.has_previous(), True)
+        self.assertEqual(page.previous_page_number(), 1)
+        self.assertEqual(page.start_index(), 10)
+        self.assertEqual(page.end_index(), 14)
+        self.assertEqual(len(page.object_list), 5)
+        self.assertEqual(page.object_list[0]['data'], 'data_04')
+
+    def tearDown(self):
+        self.conn.close()
+
+
+class TestTimeout(unittest.TestCase):
+    def setUp(self):
+        self.conn = SolrConnection(SOLR_HTTP,timeout=0.00000001)
+    
+    def test_timout_exception(self):
+        """ A socket.timeout exception should be raised 
+        """
+        
+        self.assertRaises(socket.timeout,self.conn.query,"user_id:12345")
+    
+    def tearDown(self):
+        self.conn.close()
+
 if __name__ == "__main__":
     unittest.main()
